unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList, ActnList, Menus, TB97Ctls, TB97, TB97Tlbr, FileToolbar, StdCtrls,
  ComCtrls, IniFiles;

type
  TForm1 = class(TForm)
    ActionList1: TActionList;
    FileNew: TAction;
    ImageList1: TImageList;
    FileOpen: TAction;
    FileClose: TAction;
    Dock971: TDock97;
    FileToolbar1: TFileToolbar;
    ToolbarButton971: TToolbarButton97;
    ToolbarButton972: TToolbarButton97;
    ToolbarButton973: TToolbarButton97;
    FileSave: TAction;
    FileQuit: TAction;
    FilePrint: TAction;
    EditCopy: TAction;
    EditCut: TAction;
    EditPaste: TAction;
    EditDelete: TAction;
    PopupMenu1: TPopupMenu;
    Customize1: TMenuItem;
    Label1: TLabel;
    procedure ToolbarDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ToolbarDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ToolbarEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure CallCustomizeDialog (Sender: TObject);
    procedure ActionExecute(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure PopupClick(sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
  private
  public
  end;

var
  Form1: TForm1;

implementation

uses customize;

{$R *.DFM}


//-------------------------------------------------------------------
// Dummy handler for the Actions to show that it works... 8-)
//-------------------------------------------------------------------

procedure TForm1.ActionExecute(Sender: TObject);
var act: TCustomAction;
begin
 if Sender is TAction then begin
  act:= Sender as TCustomAction;
  if Assigned (act) then begin
   MessageDlg('This is Action:'#13#10 + act.Name, mtInformation, [mbOK], 0);
  end;
 end;
end;


//-------------------------------------------------------------------
// Drag & Drop of Toolbar Buttons
// These functions are plugged to the OnDragOver, OnDragDrop and
// OnDragEnd property of any toolbar and any button on it. By user
// defined buttons and toolbars this is done automatic, by predefined
// toolbars you must plug in the functions in Objectinspector
//-------------------------------------------------------------------

procedure TForm1.ToolbarDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
// This function must be plugged to the OnDragOver property of any
// toolbars and toolbuttons
//
// Check if Dropping is allowed
begin
 if Source is TListView then
  // Docking allowed only from CustomizeDialog Command Listview
  Accept:= (Source as TListView).Name = CustomizeDialog.Commands.Name;
end;

procedure TForm1.ToolbarDragDrop(Sender, Source: TObject; X, Y: Integer);
// This function must be plugged to the OnDragDrop property of any
// toolbars and toolbuttons
//
// Handler for TToolbutton97 objects for dropping from CustomizeDialog
var t: TListView;
    act: TAction;
    ti: TListItem;
    tb, tb2: TToolbarButton97;
    ftb: TFileToolbar;
    oi: Integer;
begin
 if Source is TListView then begin // Is it from the Listview?
  t:= Source as TListView;
  if Assigned (t) then begin
   ti:= t.Selected; // Get the selected item, it holds the desired action
   if Assigned (ti) then begin
    act:= ti.Data; // The action is stored in the data property of the item
    if Assigned (act) then begin
     tb:= TToolbarButton97.Create (self); // create a toolbutton on the fly
     if Assigned (tb) then begin
      tb.Images:= ImageList1; // Assign the Imagelist
      tb.DisplayMode:= dmGlyphOnly; // display only the image
      // mode is turned to dmManual when leaving the CustomizeDialog
      // but to handle Drag&Drop, it must be dmAutomatic by now
      tb.DragMode:= dmAutomatic;
      tb.Action:= act; // the desired Action
      // Assign the handlers
      tb.OnDragDrop:= ToolbarDragDrop;
      tb.OnDragOver:= ToolbarDragOver;
      tb.OnEndDrag:= ToolbarEndDrag;
      // Hints like parent
      tb.ParentShowHint:= True;
      // Look for the place to add the button on the toolbar
      ftb:= nil;
      oi:= x div tb.Width; // Convert Position from pixel to number of buttons
      if Sender is TFileToolbar then begin // dropped direct to the toolbar?
       ftb:= Sender as TFileToolbar;
      end else if Sender is TToolbarButton97 then begin // placed on an other button?
       tb2:= Sender as TToolbarButton97;
       if Assigned (tb2) then begin // Get parent and Orderindex of the button
        ftb:= tb2.Parent as TFileToolbar;
        oi:= ftb.OrderIndex[tb2];
       end;
      end;
      if Assigned (ftb) then begin // We have a parent...
       // generate a unique name for the button
       tb.Name:= 'tbB' + CustomizeDialog.DateStamp;
       ftb.InsertControl (tb); // Insert the button on the toolbar
       ftb.OrderIndex[tb]:= oi; // and set the Orderindex
      end;
     end;
    end;
   end;
  end;
 end;
end;

procedure TForm1.ToolbarEndDrag(Sender, Target: TObject; X, Y: Integer);
// This function must be plugged to the OnEndDrag property of any
// toolbutton. The toolbars must not have this, because you cant
// throw them out of the program...
//
// Handler for TToolbarbutton97 objects to throw 'em out of the Toolbar
var tb: TToolbarButton97;
    ftb: TFileToolbar;
begin
 if not Assigned (Target) then begin // No target > so throw the button away
  tb:= Sender as TToolbarButton97;
  if Assigned (tb) then begin
   ftb:= tb.Parent as TFileToolbar;
   if Assigned (ftb) then ftb.RemoveControl (tb); // Delete the button
  end;
 end;
end;


//-------------------------------------------------------------------
// Handling the popup menu for switching toolbars on/off and
// call the customize dialog
//-------------------------------------------------------------------

procedure TForm1.PopupMenu1Popup(Sender: TObject);
// Handles the OnPopup event
// read Toolbars from Form1 and put it in the Popupmenu
var i: Integer;
    ftb: TFileToolbar;
    mi: TMenuItem;
begin
 // Clear the Popupmenu from all items
 for i:= PopupMenu1.Items.Count - 1 downto 0 do PopupMenu1.Items.Delete (i);
 // Fill all Toolbars to the menu
 for i:= 0 to ComponentCount -1 do begin
  if (Components[i] is TFileToolbar) then begin
   ftb:= Components[i] as TFileToolbar;
   mi:= NewItem (ftb.Caption, TShortCut(0), ftb.Visible, True, PopupClick, 0, 'pm' + ftb.Name);
   // Store the Address of the toolbar to the Tag property, so we can
   // access it in the OnClick handler
   mi.Tag:= Integer (ftb);
   PopupMenu1.Items.Add (mi); // Add the item
  end;
 end;
 // Add a line
 PopupMenu1.Items.Add (NewLine);
 // Add the customize item
 PopupMenu1.Items.Add (NewItem ('&Customize...', TShortCut(0),
                       False, True, CallCustomizeDialog, 0, 'pmCustomize'));
end;

procedure TForm1.PopupClick(sender: TObject);
// Switch Toolbar from visible to unvisible and back
var ftb: TFileToolbar;
begin
 // in the OnPopup handler we store the address of the toolbar
 // in the Tag property of the menuitem, so wie can simple
 // find out what toolbar to switch
 ftb:= TFileToolbar ((Sender as TMenuItem).Tag);
 if Assigned (ftb) then ftb.Visible:= not ftb.Visible;
end;

procedure TForm1.CallCustomizeDialog(Sender: TObject);
// Display CustomizeDialog. Use Show instead of Showmodal, because
// the mainform must react to the Drag&Drop messages!
begin
 CustomizeDialog.Show;
end;


//-------------------------------------------------------------------
// Saving and restoring the Toolbars
//
// Generally its a good idea to store the toolbars in a separate
// inifile away from other program data, so you can delete or
// remove it in any way. See FormClose for more details.
// If you dont like inifiles (I think for this reason, they are
// very usefull), you can store the data to the registry anyway.
// Look at the property UseRegistry of the TFileToolbar component
//-------------------------------------------------------------------

var cInifile: string; // Store path for the inifile

procedure TForm1.FormCreate(Sender: TObject);
begin
 // Get filename of the inifile, put it to program directory
 cInifile:= ExtractFilePath (Application.ExeName) + '\toolbars.ini';
end;

procedure TForm1.FormShow(Sender: TObject);
var r: TIniFile;
    sl: TStringList;
    ftb: TFileToolbar;
    i: Integer;
begin
 // Load Toolbars from inifile
 if FileExists (cInifile) then begin
  r:= TIniFile.Create (cInifile);
  sl:= TStringList.Create;
  try
   // fist read sections to get the toolbar names
   r.ReadSections (sl);
   // Loop through the sections to load the "built in" toolbars (they already
   // created with the mainform), and then generate the user defined toolbars
   for i:= 0 to sl.Count - 1 do begin
    // Load the Toolbar the program has built in
    if sl[i] = FileToolbar1.Name then
     FileToolbar1.LoadFromfile (cInifile, True)
    else begin
     // Generate the toolbars, the user has built at runtime
     ftb:= CustomizeDialog.GenToolbar (sl[i], False);
     if Assigned (ftb) then ftb.LoadFromfile (cInifile, True);
    end;
   end;
  finally
   r.Free;
   sl.Free;
  end;
  // Load toolbar positions from inifile
  IniLoadToolbarPositions (Self, cInifile, '');
 end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var ftb: TFileToolbar;
    i: Integer;
begin
 // Delete inifile to prevent us from loading old toolbars, the user
 // has deleted in this session.
 DeleteFile (cInifile);
 // Save the toolbars to inifile
 for i:= 0 to ComponentCount -1 do begin
  if (Components[i] is TFileToolbar) then begin
   (Components[i] as TFileToolbar).SaveToFile (cInifile);
  end;
 end;
 // Save the toolbar positions. Do it AFTER storing the buttons,
 // because TFileToolbar.SaveToFile erase the whole section of
 // the toolbar in the inifile
 IniSaveToolbarPositions (Self, cInifile, '');
end;

end.
