unit customize;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, CheckLst, ActnList, TB97Ctls,
  FileToolbar, TB97;

type
  TCustomizeDialog = class(TForm)
    Pagecontrol1: TPageControl;
    pcCommand: TTabSheet;
    pcToolbars: TTabSheet;
    tbNew: TButton;
    tbRename: TButton;
    tbDelete: TButton;
    CloseDialog: TButton;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Cats: TListBox;
    Label1: TLabel;
    Commands: TListView;
    Label4: TLabel;
    TbList: TListView;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CloseDialogClick(Sender: TObject);
    procedure tbNewClick(Sender: TObject);
    procedure tbRenameClick(Sender: TObject);
    procedure tbDeleteClick(Sender: TObject);
    procedure CatsClick(Sender: TObject);
    procedure CommandsChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure TbListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
   function  DateStamp: string;
   function  GenToolbar (const s: string; vis: boolean): TFileToolbar;
   procedure ButtonsDragable (mode: TDragmode);
   procedure FillToolbars;
   procedure FillCategories;
   procedure FillCommands (const cat: string);
  end;

var
  CustomizeDialog: TCustomizeDialog;

implementation

uses Unit1;

{$R *.DFM}

const csAll =     '(All)';     // All categories
      csCaption = '<Toolbar>'; // Default Toolbar Caption
      cstbU =     'tbU';       // ID for userdefined toolbars

//-------------------------------------------------------------------
// Functions for creating the TToolbars
//-------------------------------------------------------------------

function TCustomizeDialog.DateStamp: string;
// Generates an unique identifier for Toolbars and Buttons
var ts: TTimeStamp;
begin
 ts:= DateTimeToTimeStamp (Now);
 Result:= IntToStr (ts.Date) + IntToStr (ts.Time);
end;

function TCustomizeDialog.GenToolbar (const s: string; vis: Boolean): TFileToolbar;
// Generates a Toolbar at runtime
var ftb: TFileToolbar;
begin
 ftb:= TFileToolbar.Create (Form1);
 if Assigned (ftb) then begin
  ftb.Visible:= vis;
  ftb.Actions:= Form1.ActionList1; // Assign the Actionlist, very important!
  ftb.Images:= Form1.ImageList1; // Assign the imagelist
  ftb.Caption:= csCaption; // Defaultname for the toolbar
  if s <> '' then ftb.Name:= s
             else ftb.Name:= cstbU + DateStamp; // unique identifier
  // Set Position of the toolbar relative to the mainform
  ftb.Left:= Form1.Left + 50;
  ftb.Top:= Form1.Top + 100;
  // Show hints like parent
  ftb.ParentShowHint:= True;
  // Assign the DragDrop and DragOver handlers
  ftb.OnDragDrop:= Form1.ToolbarDragDrop;
  ftb.OnDragOver:= Form1.ToolbarDragOver;
  ftb.Parent:= Form1;
 end;
 Result:= ftb;
end;

//-------------------------------------------------------------------
// Functions for Drag and Drop of the Buttons
//-------------------------------------------------------------------

procedure TCustomizeDialog.ButtonsDragable (mode: TDragmode);
// Switches all TToolbarButton97 objects in Form1 into
// Dragmode "dmAutomatic", so they can be dragged from
// and to the Toolbar
var i: Integer;
    tb: TToolbarButton97;
begin
 // Loop through all components
 for i:= 0 to Form1.ComponentCount -1 do begin
  if (Form1.Components[i] is TToolbarButton97) then begin
   tb:= Form1.Components[i] as TToolbarButton97;
   if Assigned (tb) then tb.Dragmode:= mode;
  end;
 end;
end;

//-------------------------------------------------------------------
// Functions for displaying the dialog
//-------------------------------------------------------------------

procedure TCustomizeDialog.FormShow(Sender: TObject);
// prepare and display dialog
begin
 FillToolbars;
 FillCategories;
 FillCommands (csAll);
 ButtonsDragable (dmAutomatic); // Buttons dragable
end;

procedure TCustomizeDialog.FormClose(Sender: TObject; var Action: TCloseAction);
// Ready with customizing, set buttons to Dragmode "dmManual" again
begin
 ButtonsDragable (dmManual);
end;

procedure TCustomizeDialog.CloseDialogClick(Sender: TObject);
// close dialog
begin
 Close;
end;

procedure TCustomizeDialog.FillToolbars;
// read Toolbars from Form1 and put it to the Toolbar list
var i, x: Integer;
    ftb: TFileToolbar;
    ti: TListItem;
begin
 TbList.Items.BeginUpdate;
 TbList.Items.Clear; // cleat th list
 for i:= 0 to Form1.ComponentCount -1 do begin
  if (Form1.Components[i] is TFileToolbar) then begin
   ftb:= Form1.Components[i] as TFileToolbar;
   ti:= TbList.Items.Add; // Add it to the list
   if Assigned (ti) then begin
    ti.Caption:= ftb.Caption;
    // Store toolbar for easy access in later functions
    ti.Data:= ftb;
   end;
  end;
 end;
 TbList.Items.EndUpdate;
end;

procedure TCustomizeDialog.FillCategories;
// Reads the categories from the Actionlist and fills it to the Dialog
var i: integer;
    act: TCustomAction;
begin
 Cats.Items.BeginUpdate;
 Cats.Items.Clear;
 Cats.Items.Add (csAll); // Add "All" entry
 // Loop through the actions
 for i:= 0 to Form1.ActionList1.ActionCount - 1 do begin
  act:= Form1.ActionList1.Actions[i] as TCustomAction;
  if Assigned (act) then begin
   // Only categories, we dont have in the list
   if Cats.Items.IndexOf (act.Category) < 0 then begin
    Cats.Items.Add (act.Category);
   end;
  end;
 end;
 Cats.Items.EndUpdate;
end;

procedure TCustomizeDialog.FillCommands (const cat: string);
// Fills the command list with the items of the Actionlist
// cat = category that should be used. if cat = csAll, then
// fill all commands to the list
var i: integer;
    act: TCustomAction;
    ti: TListItem;

 function DelShortcutChar (const s: string): string;
 // Remove shortcut sign from caption (looks a little bit
 // boring in the List... 8-)
 var x: integer;
 begin
  Result:= s;
  x:= Pos ('&', Result);
  if x > 0 then Delete (Result, x, 1);
 end;

begin
 Commands.Items.BeginUpdate;
 Commands.Items.Clear;
 // Loop through the Actionlist
 for i:= 0 to Form1.ActionList1.ActionCount - 1 do begin
  act:= Form1.ActionList1.Actions[i] as TCustomAction;
  if Assigned (act) then begin
   // check if the category matches
   if (cat = csAll) or (cat = act.Category) then begin
    ti:= Commands.Items.Add;
    if Assigned (ti) then begin
     ti.Caption:= '';  // Leave caption epmty for better optical effect
     ti.ImageIndex:= act.ImageIndex;  // Add image
     // Add the Caption
     ti.SubItems.Add (DelShortCutChar (act.Caption)); // 0
     // The following SubItems are not shown in the Listview, but
     // stored for later access
     ti.SubItems.Add (act.Category); // 1
     ti.SubItems.Add (act.Hint); // 2
     // Store the Action in the data property, for easy access
     // in the drag&drop handler
     ti.data:= act;
    end;
   end;
  end;
 end;
 Commands.Items.EndUpdate;
end;

//-------------------------------------------------------------------
// Functions handling user action of Toolbar TabSheet
//-------------------------------------------------------------------

procedure TCustomizeDialog.tbNewClick(Sender: TObject);
// Add a new toolbar to the mainform
var ftb: TFileToolbar;
    ti: TListItem;
    x: integer;
begin
 ftb:= GenToolbar ('', true); // Generate the toolbar
 if Assigned (ftb) then begin
  ti:= TbList.Items.Add; // Add it to the list
  if Assigned (ti) then begin
   ti.Caption:= ftb.Caption;
   ti.Data:= ftb; // Store toolbar for easy access
  end;
 end;
end;

procedure TCustomizeDialog.tbRenameClick(Sender: TObject);
// Rename the toolbar
begin
 // I usually use a ListView with inplace edit capabilities
 // and Checkboxes for switch on/off Toolbars,
 // but this component is not included with this demo
 MessageDlg('If you use a Listbox with InplaceEdit'+#13+#10+'you can easy set an new name for'+#13+#10+'the toolbar.', mtInformation, [mbOK], 0);
end;

procedure TCustomizeDialog.tbDeleteClick(Sender: TObject);
// Delete toolbar
var ftb: TFileToolbar;
    ti: TListItem;
begin
 ti:= TbList.Selected;
 if Assigned (ti) then begin // delete the Toolbar
  ftb:= ti.data; // get Toolbar from data property
  if ftb <> nil then  ftb.Free; // free the toolbar
 end;
 TbList.Items.Delete (ti.Index);
end;

procedure TCustomizeDialog.CatsClick(Sender: TObject);
// Look what category the user has selected
var i: Integer;
begin
 Label3.Visible:= False;  // Hide description field
 Commands.Selected:= nil; // Delete the selection
 for i:= 0 to Cats.Items.Count - 1 do begin
  if Cats.Selected[i] then begin
   FillCommands (Cats.Items[i]); // Fill the command list
   Exit;
  end;
 end;
end;

procedure TCustomizeDialog.CommandsChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
// An Action was selected, update the description display
begin
 if Change = ctState then begin
  if Assigned (item) then begin
   // Description is the Action hint, stored in SubItems #2
   Label3.Caption:= item.SubItems[2];
   Label3.Visible:= True;
  end else begin
   Label3.Visible:= false;
  end;
 end;
end;

procedure TCustomizeDialog.TbListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
// A Toolbar was selected, update the Display
var ftb: TFileToolbar;
    b: Boolean;
begin
 if Change = ctState then begin
  if Assigned (item) then begin
   ftb:= item.data; // Get toolbar from the data property
   if Assigned (ftb) then begin
    // Let only user defined toolbars renamed or removed
    b:= Pos (cstbU, ftb.Name) = 1;
    tbRename.Enabled:= b;
    tbDelete.Enabled:= b;
   end;
  end;
 end;
end;

end.
